# This file is part of PyFd, a Python interface to the Fd library
#
# Copyright (C) 2020-2021  European Gravitational Observatory
#
# Author list: Franco Carbognani: franco.carbognani@ego-gw.it
#                   Rhys Poulton: poulton@ego-gw.it
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


import pytest
import PyFd as fd
import ctypes, time
import sys
import os

@pytest.fixture(scope="session", autouse=True)
def fdio():
    fd.CfgReachState(fd.CfgServerIdle)

    filedir =os.path.dirname(os.path.abspath(__file__)).encode("utf-8")

    cmdomain = os.environ.get("CMDOMAIN")

    if cmdomain is None:
        raise SystemExit("Error: CMDOMAIN environmental varible not set, is a NameServer running?")

    cfgfilein = b'%s/test_fdio_config.cfg.in' %filedir
    cfgfileout = b'%s/test_fdio_config.cfg' %filedir

    with open(cfgfilein,"r") as fin:
        with open(cfgfileout,"w") as fout:
            s = fin.read().replace("CM_DOMAIN",cmdomain)
            fout.write(s)

    fake_argv = [b'bla', cfgfileout, b'my_cm_name']
    arr = (ctypes.c_char_p * len(fake_argv))()
    arr[:] = fake_argv

    fdio = fd.FdIONew(len(arr), arr)
    fd.FdIOParseAndIni(fdio)

    fd.CfgReachState(fd.CfgServerConfigured)
    fd.CfgReachState(fd.CfgServerActive)

    return fdio


cm_item_getter = { # map from CmMessageGetItemType to appropriate getter
    fd.CmMessageItemTail: None,
    fd.CmMessageItemChar: fd.CmMessageGetChar,
    fd.CmMessageItemShort: fd.CmMessageGetShort,
    fd.CmMessageItemInt: fd.CmMessageGetInt,
    fd.CmMessageItemLong: fd.CmMessageGetLong,
    fd.CmMessageItemFloat: fd.CmMessageGetFloat,
    fd.CmMessageItemDouble: fd.CmMessageGetDouble,
    fd.CmMessageItemText: fd.CmMessageGetText,
    fd.CmMessageItemBytes: fd.CmMessageGetBytes,
    fd.CmMessageItemArray: fd.CmMessageGetArray,
}    

class test(object):
    __test__ = False
    def parse_cm_message(message):
        items = []
        while True:
           getter = cm_item_getter[fd.CmMessageGetItemType(message)]
           if getter is None:
              break
           item = getter(message) # error with "cm send ... -i 1.23"
           items.append(item)
        return items


    @fd.CmMessageHandler
    def my_handler(message, sender, serverName):
        print('Received message with type:', fd.CmMessageGetType(message))
        print('sender:', sender)
        print('serverName:', serverName)
        print('payload:', parse_cm_message(message))
        return fd.CmMessageBreak #or fd.CmMessageOk

def test_install_cm_handler(fdio):

    fd.CmMessageInstallHandler(test.my_handler, 'cm_type') 

    frame = fd.FdIOGetFrame(fdio)

    time.sleep(1)
    fd.FrameFree(frame)

def test_write_sms_data(fdio):

    frame = fd.FdIOGetFrame(fdio)
    
    serTime = frame.contents.GTimeS
    fsamp = 1.0 / frame.contents.dt    
    
    prefix = 'test_prefix1'
    data = 'counter 0 pi 3.1415'
    serData = fd.FrSerDataNew(frame, prefix, serTime, data, fsamp)
    try:
        assert serData.contents.data == data.encode("utf-8")
    except AssertionError as e:
        fd.FrameFree(frame)
        raise AssertionError(e)

    
    prefix = 'test_prefix2'
    data = 'counter 1 bla 123.456' 
    serData = fd.FrSerDataNew(frame, prefix, serTime, data, fsamp)
    try:
        assert serData.contents.data == data.encode("utf-8")
    except AssertionError as e:
        fd.FrameFree(frame)
        raise AssertionError(e)

    time.sleep(1)
    fd.FdIOPutFrame(fdio, frame) # no need for FrameFree?
    
    fd.CfgMsgAddUserInfo("gps of current frame %d s", serTime)
    fd.CfgMsgSendWithTimeout(.001)


def test_fdio_buff(fdio):
    frame = fd.FdIOGetFrame(fdio)

    #Get the estimated size of the frame
    f = fd.tmpfile()
    buffSize = int(1.1 * fd.FrameStat(frame,f)+ 10000)
    fd.fclose(f)

    #Create the buffer
    buff = bytes(' '*buffSize,"utf-8")

    #Write into the buffer
    nbyteswritten =  fd.FrameWriteToBuf(frame, 0, buff, buffSize, 0)

    #Check if nbyteswritten is > 0 
    if nbyteswritten==0:
        pytest.fail("There was nothing in the frame to write to the buffer")

    # Correct the size of the buffer
    buff = buff[:nbyteswritten]

    #Try to read the frame from the buffer
    frame = fd.FdIOGetFrameFromBuf(fdio,buff,nbyteswritten,-1)


